
!
!
!     =====================================================
      subroutine evec(ixy,maxm,num_eqn,num_ghost,mx,q,auxl,auxr,evl,evr)
!     =====================================================
!
!     # Calculation of left and right eigenvectors
!     # dummy file
!
      implicit double precision (a-h,o-z)
!
      write(*,*) 'Error: subroutine evec() has not been provided'
      stop
      return
      end
