num_waves = 3
num_eqn   = 4
num_aux   = 16

# Conserved quantities
depth = 0
x_momentum = 1
y_momentum = 2
z_momentum = 3

# Auxiliary variables
kappa = 0
enx = 1
eny = 2
enz = 3
etx = 4
ety = 5
etz = 6
enx = 7
eny = 8
enz = 9
etx = 10
ety = 11
etz = 12
erx = 13
ery = 14
erz = 15
