! =========================================================
subroutine tfluct1(maxnx,meqn,mwaves,maux,mbc,mx,ql,qr,auxl,auxr,amdq2)
! =========================================================
!
!	dummy tfluct function (see example euler_1d for more information)

    implicit double precision (a-h,o-z)

    write(*,*) "Error: solver.tfluct_solver=True, but you have not defined"
    write(*,*) "a tfluct solver, have you imported your solver and set solver.tfluct?"
    write(*,*) "see example/euler_1d for more information"

    stop

    return
end subroutine tfluct1


! =========================================================
subroutine tfluct2(ixyz,maxnx,meqn,mwaves,maux,mbc,mx,ql,qr,auxl,auxr,amdq2)
! =========================================================
!
!	dummy tfluct function (see example euler_1d for more information)

    implicit double precision (a-h,o-z)

    write(*,*) "Error: solver.tfluct_solver=True, but you have not defined"
    write(*,*) "a tfluct solver, have you imported your solver and set solver.tfluct?"
    write(*,*) "see example/euler_1d for more information"

    stop

    return
end subroutine tfluct2

! =========================================================
subroutine tfluct3(ixyz,maxnx,meqn,mwaves,maux,mbc,mx,ql,qr,auxl,auxr,amdq2)
! =========================================================
!
!	dummy tfluct function (see example euler_1d for more information)

    implicit double precision (a-h,o-z)

    write(*,*) "Error: solver.tfluct_solver=True, but you have not defined"
    write(*,*) "a tfluct solver, have you imported your solver and set solver.tfluct?"
    write(*,*) "see example/euler_1d for more information"

    stop

    return
end subroutine tfluct3
