#!/usr/bin/env python

# How to use this file
# python setup.py build_ext -i

import os
from os.path import join as pjoin
from os.path import pardir as pardir

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('advection_reaction_2d', parent_package, top_path)

    config.add_extension('advection_2d',
                         ['rpn2_vc_advection_vector.f90', 'rpt2_dummy.f90'])


    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())
