subroutine setprob

    !> Copy this file to your directory and modify to set up problem
    !! parameters or read other data.
    !!
    !! This default version does nothing.

    implicit none

end subroutine setprob
