num_waves = 4
num_eqn   = 4
num_aux   = 5

# Conserved quantities
depth_1 = 0
momentum_1 = 1
depth_2 = 2
momentum_2 = 3

# Auxiliary variables
bathymetry = 0
wind = 1
h_hat_1 = 2
h_hat_2 = 3
kappa = 4
