% Some useful vectors for viewing...
% Vectors to be passed to 'view' command to get
% views from top, bottom, front, back, left right.
%
% Example : To view an x-slice as if it were a 2d plot, type
%    view(xSlice)

vtop = [0,90];
vbot = [0,-90];
vfront = [0,0];
vback = [180,0];
vleft = [-90,0];
vright = [90,0];


xSlice = vright;
ySlice = vfront;
zSlice = vtop;
